@@ -32,11 +32,11 @@ Example Usage
         ctx: slash.Context, # there MUST be one argument annotated with Context
         message: msg_opt
     ):
-        """Send a message in the bot's name""" # description of command
+        """Make the bot repeat what you say""" # description of command
         # respond to the interaction, must be done within 3 seconds
         await ctx.respond(message, # string (or str()able) message
-            # sends a message without showing the command invocation
-            rtype=slash.InteractionResponseType.ChannelMessage)
+            # sends a response message immediately
+            rtype=slash.InteractionResponseType.ChannelMessageWithSource)
 
     client.run(token)
 
@@ -47,6 +47,7 @@ Notes
   completely without the involvement of messages. However, channel and author
   information is still available.
 * All descriptions are **required**.
+* You must grant the bot ``applications.commands`` permissions in the OAuth2 section of the developer dashboard.
 
 See the wiki_.
 
@@ -56,10 +57,10 @@ from __future__ import annotations
 import sys
 from warnings import warn
 from enum import IntEnum
-from typing import Coroutine, Union
+from typing import Coroutine, Union, Optional, Mapping, Any, List
 from functools import partial
+from inspect import signature
 import logging
-import datetime
 import asyncio
 import discord
 from discord.ext import commands
@@ -80,7 +81,7 @@ __all__ = [
     'SlashBot'
 ]
 
-__version__ = '0.2.3'
+__version__ = '0.5.0'
 
 class SlashWarning(UserWarning):
     """Watch out, this may cause problems down the line."""
@@ -105,31 +106,35 @@ class InteractionResponseType(IntEnum):
         Included only for completeness
     .. attribute:: Acknowledge
 
-        ACK a command without sending a message and without showing user input.
-        Probably best used for debugging commands.
+        Deprecated
     .. attribute:: ChannelMessage
 
-        Respond with a message, but don't show user input.
-        Probably best suited for admin commands,
-        or commands with ephemeral responses.
+        Deprecated
     .. attribute:: ChannelMessageWithSource
 
         Show user input and send a message. Default.
-    .. attribute:: AcknowledgeWithSource
+    .. attribute:: DeferredChannelMessageWithSource
 
-        Show user input and do nothing else.
-        Probably best suited for regular commands that require no response.
+        Show user input and display a "waiting for bot" system message.
+        Send a response with this type and edit the response later if you
+        need to do some asynchronous fetch or something.
     """
     # ACK a Ping
     Pong = 1
     # ACK a command without sending a message, eating the user's input
+    # DEPRECATED - UI shows command like a reply,
+    # so there is no way to "[eat] the user's input"
     Acknowledge = 2
     # respond with a message, eating the user's input
+    # DEPRECATED - same as Acknowledge
     ChannelMessage = 3
-    # respond with a message, showing the user's input
+    # Respond immediately to an interaction
     ChannelMessageWithSource = 4
+    # ACK an interaction and send a response later
+    DeferredChannelMessageWithSource = 5
     # ACK a command without sending a message, showing the user's input
-    AcknowledgeWithSource = 5
+    # (Former name and description, now renamed)
+    AcknowledgeWithSource = DeferredChannelMessageWithSource
 
 class MessageFlags(IntEnum):
     """Flags to pass to the ``flags`` argument of the interaction response.
@@ -155,6 +160,26 @@ class _AsyncInit:
     async def __init__(self):
         pass
 
+class PartialObject(discord.Object):
+    """Subclasses of this have their .guild as a discord.Object,
+    so their guild-related functionality may break.
+    """
+
+class PartialTextChannel(discord.TextChannel, PartialObject):
+    pass
+
+class PartialCategoryChannel(discord.CategoryChannel, PartialObject):
+    pass
+
+class PartialVoiceChannel(discord.VoiceChannel, PartialObject):
+    pass
+
+class PartialMember(discord.Member, PartialObject):
+    pass
+
+class PartialRole(discord.Role, PartialObject):
+    pass
+
 class Context(discord.Object, _AsyncInit):
     """Object representing an interaction.
 
@@ -187,41 +212,53 @@ class Context(discord.Object, _AsyncInit):
         None until interaction response has been sent
     """
 
+    id: int
+    guild: Union[discord.Guild, discord.Object, None]
+    channel: Union[discord.TextChannel, discord.Object]
+    author: Union[discord.Member, PartialMember, None]
+    command: Union[Command, Group]
+    options: Mapping[str, Any]
+    me: Union[discord.Member, discord.Object]
+    client: SlashBot
+    webhook: Optional[discord.Webhook]
+
     async def __init__(self, client: SlashBot, cmd: Command, event: dict):
         self.client = client
         self.command = cmd
         self.id = int(event['id'])
-        try:
-            self.guild = await self.client.fetch_guild(int(event['guild_id']))
-        except discord.HTTPException:
-            self.guild = discord.Object(event['guild_id'])
-            logger.debug('Fetching guild %s for interaction %s failed',
-                         self.guild.id, self.id)
-        try:
-            self.channel = await self.client.fetch_channel(int(event['channel_id']))
-        except discord.HTTPException:
-            self.channel = discord.Object(event['channel_id'])
-            logger.debug('Fetching channel %s for interaction %s failed',
-                         self.channel.id, self.id)
-        try:
-            self.author = await self.guild.fetch_member(int(event['member']['user']['id']))
-        except (discord.HTTPException, AttributeError):
-            self.author = discord.Member(
-                data=event['member'], guild=self.guild, state=self.client._connection)
-            logger.debug('Fetching member for interaction %s failed', self.id)
+        if event.get('guild_id', None):
+            self.guild = await self._try_get(
+                discord.Object(event['guild_id']), self.client.get_guild,
+                self.client.fetch_guild, 'guild')
+        else:
+            self.guild = None
+        self.channel = await self._try_get(
+            discord.Object(event['channel_id']), self.client.get_channel,
+            self.client.fetch_channel, 'channel')
+        if event.get('member', None):
+            author = PartialMember(
+                data=event['member'], guild=self.guild,
+                state=self.client._connection)
+            self.author = await self._try_get(
+                author, self._get_member,
+                self._fetch_member, 'author-member')
+        else:
+            self.author = None
         self.token = event['token']
         # construct options into function-friendly form
-        await self._kwargs_from_options(event['data'].get('options', []))
-        try:
-            self.me = await self.guild.fetch_member(self.client.user.id)
-        except (discord.HTTPException, AttributeError):
-            self.me = None
-            logger.debug('Fetching member %s (me) in guild %s '
-                         'for interaction %s failed',
-                         self.client.user.id, self.guild.id, self.id)
+        await self._kwargs_from_options(
+            event['data'].get('options', []),
+            event['data'].get('resolved', {
+                'members': {}, 'users': {},
+                'channels': {}, 'roles': {}
+            })
+        )
+        self.me = await self._try_get(
+            discord.Object(self.client.user.id), self._get_member,
+            self._fetch_member, 'me-member')
         self.webhook = None
 
-    async def _kwargs_from_options(self, options):
+    async def _kwargs_from_options(self, options, resolved):
         self.cog = self.command.cog
         kwargs = {}
         for opt in options:
@@ -235,38 +272,140 @@ class Context(discord.Object, _AsyncInit):
                     raise commands.CommandInvokeError(
                         f'No such option: {opt["name"]!r}')
                 opttype = self.command.options[opt['name']].type
+                try:
+                    opttype = ApplicationCommandOptionType(opttype)
+                except ValueError:
+                    pass # just use the new int
+                if opttype in {
+                    ApplicationCommandOptionType.USER,
+                    ApplicationCommandOptionType.CHANNEL,
+                    ApplicationCommandOptionType.ROLE,
+                }:
+                    value = discord.Object(value)
                 if opttype == ApplicationCommandOptionType.USER:
-                    try:
-                        value = await self.guild.fetch_member(int(value))
-                    except (discord.HTTPException, AttributeError):
-                        value = discord.Object(value)
-                        logger.debug('Fetching member %s for interaction %s failed',
-                                     value, self.id)
+                    def resolve_member(member):
+                        member['user'] = resolved['users'][str(value.id)]
+                        return PartialMember(
+                            data=member, guild=self.guild,
+                            state=self.client._connection)
+                    def resolve_user(user):
+                        return discord.User(
+                            state=self.client._connection, data=user)
+                    value = await self._try_get(
+                        value, self._get_member, self._fetch_member, 'member',
+                        resolve_method=resolve_member, resolved=resolved)
+                    if type(value) is discord.Object:
+                        value = await self._try_get(
+                            value, None, None, 'user', fq=False,
+                            resolve_method=resolve_user, resolved=resolved)
                 elif opttype == ApplicationCommandOptionType.CHANNEL:
-                    try:
-                        value = await self.client.fetch_channel(int(value))
-                    except (discord.HTTPException, AttributeError):
-                        value = discord.Object(value)
-                        logger.debug('Fetching channel %s for interaction %s failed',
-                                     value, self.id)
+                    def get_channel(oid):
+                        return self.guild.get_channel(oid)
+                    def resolve_channel(channel):
+                        # discord.py doesn't access this with a default,
+                        # but it seems like the resolved object doesn't
+                        # provide it either, so set it if not set.
+                        # Also can't use None here because position is
+                        # used as a sort key too.
+                        channel.setdefault('position', -1)
+                        ctype = channel['type']
+                        ctype, _ = discord.channel._channel_factory(ctype)
+                        if ctype is discord.TextChannel:
+                            ctype = PartialTextChannel
+                        elif ctype is discord.CategoryChannel:
+                            ctype = PartialCategoryChannel
+                        elif ctype is discord.VoiceChannel:
+                            ctype = PartialVoiceChannel
+                        return ctype(state=self.client._connection,
+                                     guild=self.guild, data=channel)
+                    value = await self._try_get(
+                        value, get_channel, self.client.fetch_channel, 'channel',
+                        resolve_method=resolve_channel, resolved=resolved)
                 elif opttype == ApplicationCommandOptionType.ROLE:
-                    value = self.guild.get_role(int(value))
-                    if value is None:
-                        value = discord.Object(value)
-                        logger.debug('Getting role %s for interaction %s failed',
-                                     value, self.id)
+                    def get_role(oid):
+                        return self.guild.get_role(oid)
+                    def resolve_role(role):
+                        # monkeypatch for discord.py
+                        role['permissions_new'] = role['permissions']
+                        return PartialRole(state=self.client._connection,
+                                           guild=self.guild, data=role)
+                    value = await self._try_get(
+                        value, get_role, None, 'role', fng=False,
+                        resolve_method=resolve_role, resolved=resolved)
                 kwargs[opt['name']] = value
             elif 'options' in opt:
                 self.command = self.command.slash[opt['name']]
-                await self._kwargs_from_options(opt['options'])
+                await self._kwargs_from_options(opt['options'], resolved)
                 return
         if isinstance(self.command, Group):
             self.command = self.command.slash[opt['name']]
-            await self._kwargs_from_options(opt.get('options', []))
+            await self._kwargs_from_options(opt.get('options', []), resolved)
         elif isinstance(self.command, Command):
             kwargs[self.command._ctx_arg] = self
             self.options = kwargs
 
+    async def _try_get(
+        self, default, get_method, fetch_method, typename, *,
+        resolve_method=None, resolved=None, fng=None, fq=None
+    ):
+        fq = (not self.client.resolve_not_fetch) if fq is None else fq
+        fng = self.client.fetch_if_not_get if fng is None else fng
+        # always try to get *something*
+        if fq or resolved is None \
+                or str(default.id) not in resolved[typename+'s']:
+            try:
+                obj = get_method(default.id)
+                if obj is None and fng:
+                    logger.debug(
+                        'Getting %s %s for interaction %s failed, '
+                        'falling back to fetching',
+                        typename, default.id, self.id)
+                    obj = await fetch_method(default.id)
+                elif not fng:
+                    raise ValueError
+                else:
+                    logger.debug(
+                        'Got %s %s for interaction %s',
+                        typename, obj.id, self.id)
+                    return obj
+            except discord.HTTPException:
+                logger.debug(
+                    'Fetching %s %s for interaction %s failed%s',
+                    typename, default.id, self.id,
+                    ', falling back to resolving'
+                    if resolved
+                    else ', falling back on default')
+            except (AttributeError, ValueError):
+                logger.debug(
+                    'Getting %s %s for interaction %s failed%s',
+                    typename, default.id, self.id,
+                    ', falling back to resolving'
+                    if resolved
+                    else ', falling back on default')
+            else:
+                logger.debug(
+                    'Fetched %s %s for interaction %s',
+                    typename, obj.id, self.id)
+                return obj
+        if resolved is None:
+            return default
+        obj = resolved[typename+'s'].get(str(default.id), None)
+        if obj is None:
+            logger.debug(
+                'Resolving %s %s for interaction %s failed',
+                typename, default.id, self.id)
+            return default
+        logger.debug(
+            'Resolved %s %s for interaction %s',
+            typename, default.id, self.id)
+        return resolve_method(obj)
+
+    def _get_member(self, mid):
+        return self.guild.get_member(mid)
+
+    async def _fetch_member(self, mid):
+        return await self.guild.fetch_member(mid)
+
     def __repr__(self):
         return f'<Interaction id={self.id}>'
 
@@ -291,6 +430,13 @@ class Context(discord.Object, _AsyncInit):
         rtype: :class:`InteractionResponseType`
             The type of response to send. See that class's documentation.
         """
+        if rtype in {
+            InteractionResponseType.Acknowledge,
+            InteractionResponseType.ChannelMessage,
+        }:
+            warn(f'{rtype!r} is deprecated, see: '
+                 'https://github.com/discord/discord-api-docs/pull/2615',
+                 DeprecationWarning)
         content = str(content)
         if embed and embeds:
             raise TypeError('Cannot specify both embed and embeds')
@@ -321,15 +467,14 @@ class Context(discord.Object, _AsyncInit):
             }
             if content or embeds:
                 data['data'] = {'content': content}
-            elif rtype in {InteractionResponseType.ChannelMessage,
-                        InteractionResponseType.ChannelMessageWithSource}:
+            elif rtype == InteractionResponseType.ChannelMessageWithSource:
                 raise ValueError('sending channel message with no content')
             if embeds:
                 data['data']['embeds'] = embeds
             if mentions is not None:
                 data['data']['allowed_mentions'] = mentions.to_dict()
-            if flags and 'data' in data:
-                data['data']['flags'] = int(flags)
+            if flags:
+                data.setdefault('data', {})['flags'] = int(flags)
             path = f"/interactions/{self.id}/{self.token}/callback"
             route = _Route('POST', path, channel_id=self.channel.id,
                            guild_id=self.guild.id)
@@ -352,7 +497,7 @@ class Context(discord.Object, _AsyncInit):
         Only method that works after the interaction token has expired.
         Only works if client is present there as a bot user too.
         """
-        await self.channel.send(*args, **kwargs)
+        return await self.channel.send(*args, **kwargs)
 
 Interaction = Context
 
@@ -378,23 +523,28 @@ class Option:
         Strings are converted into Choices with the same name and value
         Dicts are passed as kwargs to the Choice constructor.
     """
-    name = None
+    description: str
+    type: ApplicationCommandOptionType = ApplicationCommandOptionType.STRING
+    name: Optional[str] = None
+    required: Optional[bool] = False
+    choices: Optional[List[Choice]] = None
 
     def __init__(self, description: str,
                  type=ApplicationCommandOptionType.STRING, **kwargs):
         self.name = kwargs.pop('name', None) # can be set automatically
         self.type = type
         self.description = description
-        self.default = False #kwargs.pop('default', False)
         self.required = kwargs.pop('required', False)
-        self.choices = kwargs.pop('choices', None)
-        if self.choices is not None:
-            self.choices = [Choice.from_data(c) for c in self.choices]
+        choices = kwargs.pop('choices', None)
+        if choices is not None:
+            self.choices = [Choice.from_data(c) for c in choices]
+        else:
+            self.choices = None
 
     def __repr__(self):
         return ("Option(name={0.name!r}, type='{0.type!s}', description=..., "
-                'default={0.default}, required={0.required}, '
-                'choices={1})').format(self, '[...]' if self.choices else '[]')
+                'required={0.required}, choices={1})').format(
+                    self, '[...]' if self.choices else '[]')
 
     def to_dict(self):
         data = {
@@ -402,8 +552,6 @@ class Option:
             'name': self.name,
             'description': self.description,
         }
-        if self.default:
-            data['default'] = self.default
         if self.required:
             data['required'] = self.required
         if self.choices is not None:
@@ -423,6 +571,9 @@ class Choice:
     value: :class:`str`
         The actual value fed into the application.
     """
+    name: str
+    value: str
+
     def __init__(self, name: str, value: str):
         self.name = name
         self.value = value
@@ -441,7 +592,7 @@ class Choice:
     def to_dict(self):
         return {'name': self.name, 'value': self.value}
 
-class Command:
+class Command(discord.Object):
     """Represents a slash command.
 
     Attributes
@@ -462,35 +613,71 @@ class Command:
     options: Mapping[:class:`str`, :class:`Option`]
         Options for this command. Not passable as a parameter;
         can only be set by inspecting the function annotations.
-    check: Coroutine
-        Callback that prevents command execution
-        if it returns False (not falsy, False).
+    default: :class:`bool`
+        If ``True``, invoking the base parent of this command translates
+        into invoking this subcommand. (Not settable in arguments.)
     """
     cog = None
+    coro: Coroutine
+    id: Optional[int]
+    name: str
+    description: str
+    guild_id: Optional[int]
+    parent: Optional[Group]
+    options: Mapping[str, Option]
+    default: bool = False
 
     def __init__(self, coro: Coroutine, **kwargs):
         self.id = None
         self.name = kwargs.pop('name', coro.__name__)
         self.description = kwargs.pop('description', coro.__doc__)
+        if not self.description:
+            raise ValueError(f'Please specify a description for {self.name!r}')
         self.guild_id = kwargs.pop('guild', None)
         if self.guild_id is not None:
             self.guild_id = int(self.guild_id)
         self.parent = kwargs.pop('parent', None)
         self._ctx_arg = None
         self.options = {}
-        for arg, typ in coro.__annotations__.items():
+        found_self_arg = False
+        for param in signature(coro).parameters.values():
+            typ = param.annotation
+            if isinstance(typ, str):
+                try:
+                    # evaluate the annotation in its module's context
+                    globs = sys.modules[coro.__module__].__dict__
+                    typ = eval(typ, globs)
+                except:
+                    typ = param.empty
+            if (
+                not (isinstance(typ, Option) or (
+                    isinstance(typ, type) and issubclass(typ, Context)
+                ))
+                and param.default is param.empty
+            ):
+                if not found_self_arg:
+                    # assume that the first required non-annotated argument
+                    # is the self argument to a class' method
+                    found_self_arg = True
+                    continue
+                else:
+                    raise TypeError(
+                        f'Command {self.name!r} cannot have a '
+                        'required argument with no valid annotation')
             try:
                 if issubclass(typ, Context):
-                    self._ctx_arg = arg
+                    self._ctx_arg = param.name
             except TypeError: # not even a class
                 pass
             if isinstance(typ, Option):
                 typ = typ.clone()
-                self.options[arg] = typ
+                if param.default is param.empty:
+                    typ.required = True
+                self.options[param.name] = typ
                 if typ.name is None:
-                    typ.name = arg
+                    typ.name = param.name
         if self._ctx_arg is None:
-            raise ValueError('One argument must be type-hinted SlashContext')
+            raise ValueError('One argument must be type-hinted slash.Context')
         self.coro = coro
         async def check(*args, **kwargs):
             pass
@@ -516,6 +703,9 @@ class Command:
         }
         if self.options:
             data['options'] = [opt.to_dict() for opt in self.options.values()]
+        # TODO: the API doesn't support this yet, so it is disabled for now.
+        if self.parent is not None and False:
+            data['default'] = self.default
         return data
 
     async def invoke(self, ctx):
@@ -563,9 +753,6 @@ class Command:
         parents = []
         parent = self.parent
         while parent is not None:
-            if not parent.in_addition:
-                parent = parent.parent
-                continue
             if parent.cog is not None:
                 parents.append(partial(parent.coro, parent.cog))
             else:
@@ -582,21 +769,16 @@ class Group(Command):
     Attributes
     -----------
     coro: Coroutine
-        (Required) Callback invoked when this group is called.
-        (Base groups cannot currently be called,
-        but it will become possible at some point:
-        https://github.com/discord/discord-api-docs/issues/2393)
-    in_addition: :class:`bool`
-        If ``True``, ``coro`` is invoked before a child command is.
-        If ``False``, ``coro`` is only invoked when calling the base group.
+        (Required) Callback invoked when a subcommand of this group is called.
+        (This is not a check! Register a check using :func:`Group.check`.)
     slash: Mapping[:class:`str`, Union[:class:`Group`, :class:`Command`]]
         Subcommands of this group.
     """
     cog = None
+    slash: Mapping[str, Union[Group, Command]]
 
-    def __init__(self, coro: Coroutine, in_addition=False, **kwargs):
+    def __init__(self, coro: Coroutine, **kwargs):
         super().__init__(coro, **kwargs)
-        self.in_addition = in_addition
         self.slash = {}
 
     def slash_cmd(self, **kwargs):
@@ -627,6 +809,20 @@ class Group(Command):
         """See :class:`Group` doc"""
         self.slash_group(**kwargs)(func)
 
+    def default(self, cmd: Command):
+        """Register ``cmd`` as the default subcommand,
+        to be invoked when this base group is invoked.
+        The command must already be registered in the group.
+
+        CURRENTLY HAS NO EFFECT - default is not yet supported
+        by the API, and is therefore disabled here. This library
+        will be updated once the API is.
+        """
+        if self.slash[cmd.name] is not cmd:
+            raise ValueError(f'{cmd!s} not a subcommand of {self!s}')
+        for c in self.slash.values():
+            c.default = c is cmd
+
     def to_dict(self):
         data = {
             'name': self.name,
@@ -643,6 +839,9 @@ class Group(Command):
                 else:
                     raise ValueError(f'What is a {type(sub).__name__} doing here?')
                 data['options'].append(ddict)
+        # TODO: the API doesn't support this yet, so it is disabled for now.
+        if self.parent is not None and False:
+            data['default'] = self.default
         return data
 
 def cmd(**kwargs):
@@ -665,7 +864,9 @@ class SlashBot(commands.Bot):
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
-        self.debug_guild = int(kwargs.pop('debug_guild', 0)) or None
+        self.debug_guild = int(kwargs.pop('debug_guild', 0) or 0) or None
+        self.resolve_not_fetch = bool(kwargs.pop('resolve_not_fetch', True))
+        self.fetch_if_not_get = bool(kwargs.pop('fetch_if_not_get', False))
         self.slash = set()
         @self.listen()
         async def on_ready():
@@ -806,8 +1007,8 @@ class SlashBot(commands.Bot):
                 = {'json': todo[name].to_dict(), 'cmd': todo[name]}
         for name in to_update:
             cmd_dict = todo[name].to_dict()
-            up_to_date = all(done[name].get(k, ...) == v
-                             for k, v in cmd_dict.items())
+            up_to_date = all(done[name].get(k, ...) == cmd_dict.get(k, ...)
+                             for k in {'name', 'description', 'options'})
             if up_to_date:
                 logger.debug('GET\t%s\tin guild\t%s', name, guild_id)
                 todo[name].id = int(done[name]['id'])
